#ifndef _G4_GPIO_H
#define _G4_GPIO_H
/*
diverse Definitionen fr IO, Timer und USART2

USART2  fr ser. Ein-/Ausgaben

Timer2  32-Bit Timer; zhlt Eingangsimpulse an F1_COUNT und speichert 
        diese auf Anforderung an F1_CAPT.

Timer4  erzeugt 1 us Zeiten fr delay_us ()

Timer6  zhlt im ms-Takt die Messdauer fr F1 und F2

Timer7  erzeugt F1_TRIGGER mit F_SAMPLES typ. 200 kHz 

Timer8  zhlt die internen Referenzimpulse (Zeit) zu F2 
        TIM8_CC_IRQHandler zhlt die Eingangsimpulse an F2_CAPT

Timer15 erzeugt 1 ms-Interrupt fr LED-Blinken und Dekodierung der ext. Taster

Timer17 zhlt die internen Referenzimpulse (Zeit) zu F1
        misst ferner die ext. HSE-Frequenz bei einer nderung
*/

#define BIT(x) (1<<x)
enum gpio_modus {IN, OUT, AF, ANALOG};
enum gpio_speed {LOW_SPEED, MEDIUM_SPEED, HIGH_SPEED, V_HIGH_SPEED};
enum gpio_pullup {NO_PULL, PULL_UP, PULL_DOWN};
#define GPIO_OD       1         // open drain

#define CLR_MODER     3         // Lschmaske
#define SET_GPIO      0x1       // untere Bits
#define RESET_GPIO    0x10000   // obere Bits

#define LCD_PORT      GPIOA     // LCD und Zhlersignale
#define IIC_EE_PORT   GPIOB     // div. Ein-/Ausgnge

// Port A Verwendung der Anschlsse
#define F1_COUNT      0         // Eingang: Ereigniseingang
#define F1_CAPT       1         // Eingang: Capture-Eingang dazu

#define TxD2_Pin      2         // Ausgang: USART2
#define RxD2_Pin      3         // Eingang: dto
#define KONTRAST      4         // Ausgang: DAC1-Ausgang fr LCD-Kontrast

#define LCD_CMD       5         // Ausgang: 0 = CMD, 1 = Zeichenausgabe
#define LCD_READ      6         // Ausgang: 0 = Schreiben, 1 = Status lesen
#define LCD_STROBE    7         // Ausgang: Schreibimpuls E fuers LCD
#define LCD_D4        8         // Datenleitungen / Taster
#define LCD_D5        9
#define LCD_D6        10
#define LCD_D7        11

#define LCD_DATA_OFFSET LCD_D4  // 4 Bit ab hier ausgeben

#define F1_TRIGGER    12        // Ausgang: Auswertung anfordern
#define F2_CAPT       15        // Eingang: Ereignisse von 1 pps-Signal

// Port B
#define EXT_REF       0         // Eingang: ext. Ref. aktiv
#define Q0_EINGANG    3         // Eingang: Einlesen des Vorteilers
#define LED_FERTIG    4         // Ausgang: blinkt bei fertiger Messung
#define F1_TIME_CAPT  5         // Eingang: Zeitmessung TIM3_CH2
#define SDA_PIN       6         // IIC_EE
#define SCL_PIN       7
#define EX_REF_ON     8         // Ausgang: 1 = ext. Ref. verwenden

// Port F
#define LED_1PPS      1         // Ausgang: Dauerlicht, wenn locked

// Alternative Funktionen und Interrupt-Prioritten USART und Timer
#define PRIO_USART2   10        // mittlere Prioritt
#define AFR_USART2    7         // fr rx/tx lt. Datenblatt

#define PRIO_TIM15    14        // zum Dekodieren der Taster
#define PRIO_TIM17    4         // F1-Capture, Zeitmessung            

#define PRIO_TIM6     14        // 1 kHz Timer unkritisch
#define PRIO_TIM7     10        // < PRIO_TIM17 sonst unkritisch
#define AFR_TIM2      1         // fr T2  lt. Datenblatt
#define AFR_TIM17     10        // fr T17 lt. Datenblatt

#define PRIO_TIM8     5         // fr 1pps-Eingang unkritisch
#define AFR_TIM8      2         // Capture TIM8


#endif