#define FMETER_G431
/* 
Hauptprogramm mit Endlosschleife fr Messungen und Bedienung ohne return zu main()

Beim Programmstart ist der ungenaue, interne HSI-Takt aktiv. Dieser ist nur
fr die Initialisierung der Variablen, ser. Schnittstelle und LCD brauchbar.
Nach Anzeige der Programmversion mu zunchst getestet werden, ob der 
lokale 10 MHz VCTCXO-Takt oder ein externer Referenztakt verwendet werden soll.
Danach sind die eigentlichen Messungen mglich.

Die externe Referenzfrequenz kann im laufenden Betrieb zu- und abgeschaltet
werden. Beim Zuschalten erfolgt eine "weiche" Umschaltung auf die ext. Referenz.
Beim Abschalten fehlt dem C die Taktfrequenz, weshalb prozessorintern auf HSI-Takt
zurckgeschaltet und ein NMI ausgelst werden. Per NMI_Handler() wird der Systemtakt
wieder auf die "soll_frequenz" gestellt und mit NMI_flag signalisiert, dass
ggf. ein manuelles Bedienmenue verlassen wird.
Nach Rckkehr ins Hauptprogramm wird dann erneut nach gltiger HSE-Taktquelle
gesucht und diese wie beim Einschalten aktiviert.


Die Messungen der beiden Eingnge F1/F2 werden einmalig gestartet und laufen dann
per Interrupt im Hintergrund. Mit dem Aufruf von lese_taste() wird zum einen 
geprft, ob Einstellwerte manuell verndert werden sollen, und zum anderen
permanent die Funktionen F1_messung(0) und F2_messung(0) aufgerufen. Diese
Aufrufe testen, ob neue Messwerte vorliegen und werten sie aus bzw. zeigen
die Ergebnisse an.

experimentelle bertaktung:
Mit der Funktion turbo_test() wird getestet, ob die LCD-Leitungen RS und R/W
mit einem 3k3 Widerstand verbunden sind. Fall ja, wird eine '1' als Ergebnis
geliefert, woraufhin die soll_frequenz auf einen hheren Wert eingestellt wird.
Dadurch erhht sich die max. Messfrequenz auf 200 MHz.

2021-03-16
Michael Nowak
http://www.mino-elektronik.de
Alle Angaben ohne Gewaehr !
*/

#include "system_G431.h"
uint16_t  HSE_setup_us;                     // HSE Einschwingzeit
uint8_t   ex_ref_aktiv = 0;                 // letzter Zustand zur einfachen Abfrage
uint8_t   NMI_flag;                         // fr Abbruch manueller Einstellung bei Taktausfall
int32_t   F1_offset  = DEF_OFFSET;          // Default-Werte fr 1. Einschalten
int32_t   Fintern_offset  = DEF_OFFSET;     // dto.
int32_t   Fextern_offset  = DEF_OFFSET;     // dto.
uint32_t  abgleich_zeit  = DEF_GPS_FILTER;  // dto.
uint32_t  abgleich_zeit_intern  = DEF_GPS_FIL_INT;  // typ. 100 s
uint32_t  abgleich_zeit_extern  = DEF_GPS_FILTER;   // extern 10 Minuten
uint32_t  hse_value;
double    F_ref;                            // genauer Wert der PLL-Frequenz

uint8_t anzeige_funktion = 0,               // beim 1. Einschalten alle mit Standardwert starten
        anzeige_format = 0,                 // werden dann mit gltigen EEPROM-Werten berschrieben
        nur_MHz = 0,
        F1_reg_modus = OPTIMIERT,           // standard: aktiv
        ser_ausgabe_kanal = F1;             // F1 ausgeben

               //12345678901234567890 
char mino[] =  {"      mino      "};
char version[]={"FMETER-G431 V1.4"};

uint32_t soll_frequenz = STD_TAKTFREQUENZ;  // std oder turbo

// Test auf 3k3 Widerstand zwischen RS und R/W-Pin
uint8_t turbo_test(void)                      
{
volatile uint32_t temp;
  RCC->AHB2ENR |= RCC_AHB2ENR_GPIOAEN;        // PortA aktivieren
  LCD_PORT->MODER &= ~(CLR_MODER << LCD_CMD*2 | 
                       CLR_MODER << LCD_READ*2);
  LCD_PORT->MODER |=  (IN << LCD_CMD*2 |      // RS auf Eingang, R/W auf Ausgang
                       OUT << LCD_READ*2);
  LCD_PORT->PUPDR |= (PULL_UP << LCD_CMD*2);  // RS mit Pullup
  for(temp = 0; temp < 1000; temp++);         // kleine Warteschleife
  if(LCD_PORT->IDR & BIT(LCD_CMD)) return(0); // Pin auf '1': kein Widerstand vorhanden
  set_pin(LCD_PORT, LCD_READ, 1);             // R/W auf '1' setzen
  for(temp = 0; temp < 1000; temp++);         // kleine Warteschleife
  if(LCD_PORT->IDR & BIT(LCD_CMD)) return(1); // Pin jetzt auf '1': Widerstand vorhanden
  return(0);                                  // Pegel an RS folgt nicht dem Pegel an R/W
}  
  
  
// HSI auf soll_frequenz einstellen
void HSI_einschalten(void)
{
uint32_t pll_n;
  pll_n = (soll_frequenz / MHZ_FAKTOR) *                // berlauf vermeiden
           HSI16_PLL_M * PLL_R / (HSI_VALUE / MHZ_FAKTOR);
  
  RCC->CICR = RCC_CICR_CSSC;                            // CSSC Flag lschen
  RCC->CR |= RCC_CR_HSION;                              // fr internen 16 MHz takt einstellen
  while((RCC->CR & RCC_CR_HSIRDY) == 0);                // Fertig-Bit abwarten
  RCC->CFGR &= (uint32_t)~(RCC_CFGR_SW_1);              // PLL nicht verwenden
  RCC->CR &= ~RCC_CR_PLLON;                             // und dann abschalten
  RCC->PLLCFGR = RCC_PLLCFGR_PLLREN |
                 pll_n << RCC_PLLCFGR_PLLN_Pos |
                 (HSI16_PLL_M-1) << RCC_PLLCFGR_PLLM_Pos |
                 RCC_PLLCFGR_PLLSRC_HSI;  
  RCC->CR |= RCC_CR_PLLON;                              // PLL wieder einschalten
  while((RCC->CR & RCC_CR_PLLRDY) == 0);                // Fertig-Bit abwarten
  RCC->CFGR |= RCC_CFGR_SW_PLL;                         // PLL als Haupttakt whlen
  while ((RCC->CFGR & (uint32_t)RCC_CFGR_SWS ) != RCC_CFGR_SWS_PLL);  // warten bis aktiv
  F_ref = (double)HSI_VALUE * pll_n / (HSI16_PLL_M) / PLL_R;
  SystemCoreClock = (uint32_t)F_ref;
}  

// HSE Signal auf ganze MHz messen
// Ergebnis in hse_value; ggf. 0, falls kein Takt vorhanden ist
uint32_t HSE_testen(void)
{
volatile uint32_t timeout_cnt = 10000;  
uint16_t  HSE_status, cap_wert=0;         // Zwischenwert zum Messen
uint8_t pll_m = HSI16_PLL_M;              // default wie per #define  

// Reset the RCC clock configuration to the default reset state
  RCC->CR |= RCC_CR_HSION;                // HSI einschalten
  while((RCC->CR & RCC_CR_HSIRDY) == 0);  // Fertig-Bit abwarten
  RCC->CFGR = RCC_CFGR_SW_HSI;            // HSI als Taktquelle
  RCC->CR &= RCC_CR_HSIRDY | RCC_CR_HSION;// Reset HSEON, CSSON and PLLON bits
  RCC->PLLCFGR = 0x00001000;              // Reset PLLCFGR register lt. Datenblatt
  RCC->CR |= RCC_CR_HSEBYP;               // Set HSEBYP bit
  RCC->CIER = 0x00000000;                 // Disable Pll interrupts

// Versuchen, HSE-Takt zu messen
  RCC->CR |= ((uint32_t)RCC_CR_HSEON);    // Enable HSE
  do {                                    // Wait till HSE is ready or Timeout
    HSE_status = RCC->CR & RCC_CR_HSERDY;
  } while((HSE_status == 0) && (timeout_cnt--));
  if(timeout_cnt) {                       // kein Timeout erreicht
    RCC->CR &= ~RCC_CR_PLLON;             // und dann abschalten
// zunaechst mit HSI-Frequenz (16MHz) eff. HSE-Takt ermitteln
    RCC->APB2ENR |= RCC_APB2ENR_TIM17EN;  // Timer17 aktivieren fr HSE-Messung
    TIM17->CR1 = 0;;                      // T17 stoppen
    TIM17->DIER = 0;                      // und alle Interrupts abschalten
    TIM17->CCMR1 = TIM_CCMR1_CC1S_0 + TIM_CCMR1_IC1PSC_1 + TIM_CCMR1_IC1PSC_0;   // capture zuordnung IC1 -> TI7, prescaler /8
    TIM17->CCER |= TIM_CCER_CC1E; 
    TIM17->OR = TIM_OR_HSE32EN;           // HSE_RTC-Takt als capture-Signal; Teiler: 32*8 = 256
    TIM17->TISEL = TIM_TISEL_TI1SEL_1 + TIM_TISEL_TI1SEL_0;
    TIM17->CNT = 0;
    TIM17->CR1 |= TIM_CR1_CEN;            // T17 starten
    while(!(TIM17->SR & TIM_SR_UIF));     // einmalig Ueberlauf abwarten
    TIM17->SR = 0;                        // UIF und CC1F-Flags loeschen
    while(!(TIM17->SR & TIM_SR_CC1IF) && !( TIM17->SR & TIM_SR_UIF));     // capture abwarten
    if(TIM17->SR & TIM_SR_UIF) {
      hse_value = 0;                      // kein ext. Takt vorhanden
      HSE_status = 0;
      pll_m = HSI16_PLL_M;
    } else {
      HSE_status = 1;
      cap_wert = TIM17->CCR1;             // 1. wert lesen
      TIM17->SR = 0;                      // UIF und CC1F-Flags loeschen
      while(!(TIM17->SR & TIM_SR_CC1IF)); // capture abwarten
      cap_wert = TIM17->CCR1 - cap_wert;  // differenz bilden
      pll_m = PLL_M_DIVIDEND / cap_wert;  // mit 2% aufrundet wg. Toleranz
      hse_value = pll_m * MHZ_FAKTOR;     // entsprechend anpassen
    }  

    RCC->APB2RSTR |= RCC_APB2RSTR_TIM17RST; // Timer17 lschen
    RCC->APB2RSTR &= ~RCC_APB2RSTR_TIM17RST;
    RCC->APB2ENR &= ~RCC_APB2ENR_TIM17EN;   // Timer17 wieder abschalten
    return(hse_value);
  }
  return(0);                              // bei Timeout HSE = 0 MHz
}

void berechne_pllm_plln(uint32_t *pll_m, uint32_t *pll_n, uint32_t hse, uint32_t f_soll)
{
uint32_t temp;  
    if(hse == 10000000 && soll_frequenz == STD_TAKTFREQUENZ) {
      *pll_m = 3;                                        // gezielt "krummer" Wert 
    } else {  
      *pll_m = 1;
      if(hse >=  8000000) (*pll_m)++;
      if(hse >= 12000000) (*pll_m)++;
      if(hse >= 16000000) (*pll_m)++;
      if(hse >= 20000000) (*pll_m)++;
    }
    temp = (f_soll / MHZ_FAKTOR) *              // berlauf vermeiden
             *pll_m * PLL_R / (hse/ MHZ_FAKTOR);
    *pll_n = temp;
}

void aktiviere_HSE_pll(uint32_t pll_m, uint32_t pll_n, uint32_t hse)
{
volatile uint32_t timeout_cnt = 10000;                  // grobe Vorgabe zum Start der PLL
  RCC->CICR = RCC_CICR_CSSC;                            // CSSC Flag lschen
  RCC->CR |= ((uint32_t)RCC_CR_HSEON);                  // Enable HSE
  do {                                                  // Wait till HSE is ready or Timeout
  } while((RCC->CR & RCC_CR_HSERDY == 0) && (timeout_cnt--));
  if(timeout_cnt) {                                     // kein Timeout erreicht

    RCC->CFGR &= ((uint32_t)~(RCC_CFGR_SW_1));          // PLL nicht verwenden
    RCC->CR &= ~RCC_CR_PLLON;                           // und dann abschalten
    RCC->PLLCFGR = RCC_PLLCFGR_PLLREN |
                  pll_n << RCC_PLLCFGR_PLLN_Pos |
                  (pll_m-1) << RCC_PLLCFGR_PLLM_Pos |
                  RCC_PLLCFGR_PLLSRC_HSE;  
    RCC->CR |= RCC_CR_HSEBYP;                           // HSEBYP bit
    RCC->CR |= RCC_CR_PLLON;                            // PLL wieder einschalten
    while((RCC->CR & RCC_CR_PLLRDY) == 0);              // Fertig-Bit abwarten
    RCC->CFGR = (RCC->CFGR & ~(RCC_CFGR_SW)) | RCC_CFGR_SW_PLL;  // PLL als Haupttakt whlen
    while ((RCC->CFGR & (uint32_t)RCC_CFGR_SWS ) != RCC_CFGR_SWS_PLL);  // warten bis aktiv
    F_ref = (double)hse * pll_n / pll_m / PLL_R;
    SystemCoreClock = (uint32_t)F_ref;
  }  
}  


// PLL auf soll_frequenz einstellen
// und HSE aktivieren
void HSE_einschalten(uint32_t hse_takt, uint32_t f_soll)
{
uint32_t pll_m, pll_n; 
  TIM15->CNT = 0;                                       // als s-Timer benutzen
  berechne_pllm_plln(&pll_m, &pll_n, hse_takt, f_soll);
  aktiviere_HSE_pll(pll_m, pll_n, hse_takt);
  HSE_setup_us = TIM15->CNT;                            // Zeit in s fr Aktivierung HSE-PLL
}  

// testen, ob DC-Signal (von Ladungspumpe) anliegt
uint8_t teste_ex_ref(void)
{
  if(GPIOB->IDR & BIT(EXT_REF)) return(1); // ext. Ref. vorhanden
  return(0);
}

// testen, ob Multiplexer schon auf ext.Ref. steht
uint8_t teste_akt_ref(void)
{
  if(GPIOB->IDR & BIT(EX_REF_ON))
    return(1);
  return(0);                              // 0 = lokale Referenz
} 

// Versionsanzeige beim Einschalten
void name()
{
  lcd_zeile1(mino);
  lcd_zeile2(version);
}

// wenn ext. Referenz vorhanden diese einschalten
// sonst lokalen VCTCXO verwenden
void aktiviere_referenz(uint8_t ex_ref)
{
  uint32_t temp;
	RCC->AHB2ENR |= RCC_AHB2ENR_GPIOBEN;                // PortB aktivieren
  temp = GPIOB->MODER;
  temp &= ~((CLR_MODER << (EXT_REF * 2) |
             CLR_MODER << (EX_REF_ON * 2)));                       
  temp |= (IN << (EXT_REF * 2) |                      // als Eingang
           OUT << (EX_REF_ON * 2));                   // Ausgang fr Multiplexer
  GPIOB->MODER = temp;
  
  if(ex_ref) {
    set_pin(GPIOB, EX_REF_ON,1);                      // aktivieren mit '1'
    ex_ref_aktiv = 1;
  } else {
    set_pin(GPIOB, EX_REF_ON,0);                      // lokale Ref. mit '0'
    ex_ref_aktiv = 0;
  }  
}

// F1-Taktfrequenz fr lokalen/externen Takt errechnen
void berechne_F1_takt()
{
  if(ex_ref_aktiv == 0) {
    F1_offset = Fintern_offset;
    abgleich_zeit = abgleich_zeit_intern;
  } else {
    F1_offset = Fextern_offset;
    abgleich_zeit = abgleich_zeit_extern;
  } 
  F1_taktfrequenz = F_ref + 
    (double)F1_offset * F_ref * F_OFFSET_SCHRITT;     // in 0,01 ppb Schritten
}       

// bei Ausfall von HSE wird auf HSI umgeschaltet
// ext. NMI-Handler mu ausgeklammert werden
void NMI_Handler(void)
{
  HSI_einschalten();
  NMI_flag = 1;                   // ggf. Bedienmenue abbrechen
}

// Hauptprogramm: Aufruf ohne return aus main()
int fmeter_g431(void) 
{
uint8_t ee_init, c;
uint32_t temp;
char s[20];

// Option 240 MHz testen und ggf. einschalten
  soll_frequenz = STD_TAKTFREQUENZ;
  if(turbo_test()) {                    // direkt beim Programmstart: testet auf ext. Widerstand
    soll_frequenz = TURBO_TAKTFREQUENZ;
    temp = FLASH->ACR;
    temp &= ~FLASH_ACR_LATENCY_Msk;    
    temp |= TURBO_FLASH_ZUGRIFF;
    FLASH->ACR = temp;                                  // passend zu 240 MHz
  }  
  HSI_einschalten();                                    // aktuelle soll_frequenz verwenden
  
  delay_ms(100);                                        // stabile Versorgungsspannung abwarten
  
// gespeicherte Einstellungen laden; Default-Werte beim 1. Einschalten
  ee_read_uint8(EE_ADR_PARA_INIT, &ee_init);
    if(ee_init == EE_INIT_MUSTER) {                     // EE-Muster ist gltig
      lese_ee_F1_parameter();                           // dann alte Einstellungen
      lese_ee_F2_parameter();                           // zurcklesen
      lese_ee_allgemein_parameter();
    } else {
      ee_write_uint8(EE_ADR_PARA_INIT, EE_INIT_MUSTER); // Kennung schreiben
      schreibe_ee_F1_parameter();                       // und danach Defaultwerte
      schreibe_ee_F2_parameter();                       // fr beide Kanle
      schreibe_ee_allgemein_parameter();
    }  

  init_t15();                                   // Bedientaster abfragen
  init_usart2();
  lese_baudraten_index();                       // eff. Baudrate einstellen
  lcd_init();                                   // immer zuerst LCD versuchen
  
  init_zeilen_breite(lcd_breite);               // Tabelle initialisieren
  setze_kontrast(lcd_kontrast);
  name();
  delay_ms(2000);
// Fehlender HSE-Takt mu NMI auslsen  
  RCC->CR |= (RCC_CR_CSSON);                    // CSSON aktivieren

  while(1) {
    if(!(RCC->PLLCFGR & RCC_PLLCFGR_PLLSRC_0)) {// wenn HSE fehlt
      aktiviere_referenz(0);                    // zunchst interne Fref verwenden
      lcd_clear();
      lcd_zeile1("      STOPP!      ");         
      lcd_zeile2("  Referenz fehlt  ");
      while(!HSE_testen());                     // auf HSE warten: Hardware defekt!
      if(teste_ex_ref()) aktiviere_referenz(1); // ext. Takt, wenn vorhanden
      lcd_clear();
      HSE_testen();                             // hse_value messen
      if(hse_value) {
        HSE_einschalten(hse_value, soll_frequenz);             // HSE verwenden
      }  
      if(teste_ex_ref())
        lcd_zeile1("ext-Ref. erkannt");
      else
        lcd_zeile1("int-Ref. erkannt");
      wandel_Fx("",hse_value, 0, s, 10);        // akt. Wert anzeigen
      lcd_zeile2(s);
      delay_ms(1000);
    }

// wenn HSE erkannt und aktiviert    
    berechne_F1_takt();                         // offset mit SystemCoreClock verrechen
    gps_fil_laenge = abgleich_zeit;             // Intervall fr Mittelwertbildung
    loesche_gps_filter();                       // und Messung neu starten

// Nach nderung von HSE: Messungen neu initialisieren    
    do {  
      set_baudrate2(baud_tab[baudraten_index]); // falls SystemCoreClock verndert wurde
      F1_messung(1);                            // beide Kanle einschalten
      F2_messung(1);
// Dauerschleife, bis sich HSE-Takt verndert oder ausfllt    
      while(1) { 
        c = lese_taste();                       // F1 + F2 Messungen werden hier aufgerufen
        if(c == PLUS_TASTE) {                   // angezeigte Werte direkt umschalten
          if(anzeige_funktion < MAX_FUNKTION-1) {
            anzeige_funktion++;
            ee_write_uint8(EE_ADR_ANZEIGE_FKT, anzeige_funktion);
          }
        } else if(c == MINUS_TASTE) {           // angezeigte Werte direkt umschalten
          if(anzeige_funktion > 0) {
            anzeige_funktion--;
            ee_write_uint8(EE_ADR_ANZEIGE_FKT, anzeige_funktion);
          }
        } else if(c == INIT_TASTE) {            // Bedienmenue aufrufen
          ausgabe_sperre = 1;                   // Messwertanzeige sperren
          G431_hauptmenue();
          ausgabe_sperre = 0;                   // wieder zulassen
          update_header = 1;                    // und Anzeige aktualisieren
        }  
        if(teste_ex_ref() != teste_akt_ref()) { // ext. Ref. zugeschaltet
          HSI_einschalten();                    // Referenz mit HSI-Takt neu suchen
        }  
        if(!(RCC->PLLCFGR & RCC_PLLCFGR_PLLSRC_0)) break; // HSE fehlt: Messungen abbrechen
      }
    } while((RCC->PLLCFGR & RCC_PLLCFGR_PLLSRC_0));       // HSE fehlt: Referenz neu ermitteln
  }   // HSE fehlt, neue Taktquelle suchen+aktivieren
}