#define G431_EE_IIC
/*
2020-12-10
Michael Nowak
http://www.mino-elektronik.de
Alle Angaben ohne Gewaehr !
*/

#include "system_G431.h"

#define WR_ADR_ACK          0x1     // status bei gueltiger adressierung
#define DEF_EE_BASIS_ADR    0xa0    // hardware Einstellung
#define DEF_MULTI_BYTE      8       // mehrere Bytes in einem Zyklus schreiben

static uint8_t ee_base_adr = DEF_EE_BASIS_ADR,
               multi_byte = DEF_MULTI_BYTE;
         

void kurz_warten()
{
  delay_us(1);      // fuer ca. 200kHz IIC-Takt   
}


uint8_t read_scl()
{
  kurz_warten();
  if(IIC_EE_PORT->IDR & (1<< SCL_PIN)) return(1);
  return(0);
}

uint8_t read_sda()
{
  kurz_warten();
  if(IIC_EE_PORT->IDR & (1<< SDA_PIN)) return(1);
  return(0);
}


void scl_0()                    // subroutinen fuer langsame Ausfuehrung
{
  set_pin(IIC_EE_PORT, SCL_PIN, 0);  // loeschen
  kurz_warten();
}

void scl_1()
{
  set_pin(IIC_EE_PORT, SCL_PIN, 1);  // setzen
  while(!read_scl());          // warten bis scl = 1
  kurz_warten();
}

void sda_0()
{
  set_pin(IIC_EE_PORT, SDA_PIN, 0);  // loeschen
  kurz_warten();
}

void sda_1()
{
  set_pin(IIC_EE_PORT, SDA_PIN, 1);  // setzen
  kurz_warten();
}

void iic_start()
{
static uint8_t init = 0;
uint32_t tmp;

  if(!init) {
    RCC->AHB2ENR |= RCC_AHB2ENR_GPIOBEN;        // PortB aktivieren
    init = 1;
// Ausgnge fr IIC_EE: Open Drain mit PullUp 
    tmp = IIC_EE_PORT->MODER & ~(CLR_MODER << (SDA_PIN * 2) | CLR_MODER << (SCL_PIN * 2));
    IIC_EE_PORT->MODER = tmp | (OUT << (SDA_PIN * 2) | OUT << (SCL_PIN * 2)); 
    IIC_EE_PORT->OTYPER |= (GPIO_OD << (SDA_PIN) | GPIO_OD << (SCL_PIN)); 
    IIC_EE_PORT->PUPDR |= (PULL_UP << SDA_PIN*2 | PULL_UP << SCL_PIN*2);
    IIC_EE_PORT->OSPEEDR |= (HIGH_SPEED << SDA_PIN*2 | HIGH_SPEED << SCL_PIN*2);
  }
  scl_0();              // zunaechst Clock auf 0
  sda_1();              // damit keine Stopkondition entstehen kann
  scl_1();              // sda und scl sind passiv
  sda_0();              // Startbedingung
}

void iic_stop()
{
  sda_0();              // muss unten sein
  scl_1();              // Clock zuerst auf 1
  sda_1();              // Stopbedingung
}

void set_ack()
{
  scl_0();              // auf 0, damit keine Startbedingung
  sda_0();              // Datenbit auf 0 -> ack
  scl_1();              // impuls dazu
  scl_0();              // ack. ist vorbei
  sda_1();              // wieder auf eingang schalten
}

void set_nack()
{
  scl_0();              // auf 0, damit keine Startbedingung
  sda_1();              // Datenbit auf 1 -> ack
  scl_1();              // impuls dazu
  scl_0();              // ack. ist vorbei
}

uint8_t get_ack()       // ack-impuls ist activ low
{
uint8_t temp;
  scl_0();              // auf 0, damit keine Startbedingung
  sda_1();              // Datenbit auf 1 -> ack
  scl_1();              // impuls dazu
  temp = read_sda();    // falls benoetigt
  scl_0();              // ack. ist vorbei
  return(temp ^ 1);     // 1 = ack-signal erhalten
}

uint8_t iic_write(uint8_t wert)
{
uint8_t n;
  for(n=0; n<8; n++) {
    scl_0();            // Aenderungen nur bei clock=0
    if(wert & 0x80) sda_1();
    else sda_0();
    scl_1();            // clock dazu
    wert <<= 1;         // von msb -> lsb
  }
  return(get_ack());
}

uint8_t iic_read_ack(uint8_t *wert)
{
uint8_t temp,n;
  scl_0();              // clock muss 0 sein
  sda_1();              // sda als input
  for(n=0; n<8; n++) {
    temp <<= 1;         // von msb -> lsb
    scl_1();            // daten einlesen
    temp += read_sda();
    scl_0();            // clock wegnehmen
  }
  *wert = temp;         // gelesenes byte ablegen
  set_ack();            // weitere bytes werden erwartet
  return(0);            // dummy status
}

uint8_t iic_read_nack(uint8_t *wert)
{
uint8_t temp,n;
  scl_0();              // clock muss 0 sein
  sda_1();              // sda als input
  for(n=0; n<8; n++) {
    temp <<= 1;         // von msb -> lsb
    scl_1();            // daten einlesen
    temp += read_sda();
    scl_0();            // clock wegnehemen
  }
  *wert = temp;         // gelesenes byte ablegen
  set_nack();           // transfer beenden
  return(0);            // dummy status
}

uint8_t iic_adr_write(uint8_t adr)  // baustein zum schreiben adressieren
{
  iic_start();
  return(iic_write(adr & 0xfe));    // return(1), wenn baustein vorhanden
}

uint8_t iic_adr_read(uint8_t adr)   // baustein zum lesen adressieren
{
  iic_start();
  return(iic_write(adr | 1));       // return(1), wenn baustein vorhanden
}

void init_ee_base(uint8_t basis, uint8_t multi)
{
  ee_base_adr = basis;
  multi_byte = multi;
}

static void form_page_adr(uint16_t adr, uint8_t *page, uint8_t *rest)
{
  *page = ee_base_adr + ((adr/256) << 1);   // IIC-Adresse bilden
  *rest = adr & 0xff;                       // rest der gesamtadresse
}

uint8_t iic_test(uint8_t iic_adr)       // Baustein testen
{
uint8_t status;
int timeout=500;                        // wartet ca. 10ms maximal
  while(timeout--) {
    status = iic_adr_write(iic_adr);    // nur die Basisadresse testen
    iic_stop();
    if(status == WR_ADR_ACK) return(0); // baustein laesst sich ansprechen
  }
  return(1);                            // laesst sich nicht ansprechen
}

uint8_t ee_write_n(uint16_t adr, void *ptr, uint16_t anzahl)    // array schreiben
{
uint8_t page, rest, status=0, n;
uint8_t *p = ptr;
  while(anzahl) {
    n = multi_byte - (adr & (multi_byte-1));    // adr ausrichten
    form_page_adr(adr, &page, &rest);
    if(n > anzahl) n = anzahl;          // begrenzen
    anzahl -= n;
    adr += n;                           // adresse fuer naechsten zugriff
    if(status = iic_test(page))         // test, ob eeprom bereit
      return(status);                   // eeprom nicht vorhanden
    iic_adr_write(page);                // eeprom adressieren
    iic_write(rest);                    // zieladresse uebergeben
    for(;n;n--)
      iic_write(*(p++));                // bytes schreiben
    iic_stop();                         // Programmierung starten
  }
  return(status);
}

uint8_t ee_read_n(uint16_t adr, void *ptr, uint16_t anzahl) // array lesen
{
uint8_t page, rest, status=0;
uint8_t *p = ptr;
  if(!anzahl--) return(1);              // anzahl = 0
  form_page_adr(adr, &page, &rest);
  if(status = iic_test(page))           // test, ob eeprom bereit
    return(status);                     // eeprom nicht vorhanden
  iic_adr_write(page);                  // adressieren
  iic_write(rest);                      // leseadresse vorgeben
  iic_adr_read(page);                   // und read-modus einstellen
  while(anzahl--) {
    iic_read_ack(p);                    // anzahl-1 lesen
    p++;        
  }
  iic_read_nack(p);                     // letztes byte ohne Quittung
  iic_stop();
  return(status);
}


uint8_t ee_write_uint8(uint16_t adr, uint8_t wert)      // ein byte schreiben
{
  return(ee_write_n(adr, &wert, sizeof(char)));
}

uint8_t ee_write_uint16(uint16_t adr, uint16_t wert)    // zwei byte schreiben
{
  return(ee_write_n(adr, &wert, sizeof(uint16_t)));
}

uint8_t ee_write_uint32(uint16_t adr, uint32_t wert)  // vier byte schreiben
{
  return(ee_write_n(adr, &wert, sizeof(uint32_t)));
}

uint8_t ee_write_double(uint16_t adr, double wert)  // acht byte schreiben
{
  return(ee_write_n(adr, &wert, sizeof(double)));
}


uint8_t ee_read_uint8(uint16_t adr, uint8_t *wert)      // ein byte lesen
{
  return(ee_read_n(adr, wert, sizeof(uint8_t)));
}

uint8_t ee_read_uint16(uint16_t adr, uint16_t *wert)    // zwei byte lesen
{
  return(ee_read_n(adr, wert, sizeof(uint16_t)));
}

uint8_t ee_read_uint32(uint16_t adr, uint32_t *wert)    // vier byte lesen
{
  return(ee_read_n(adr, wert, sizeof(uint32_t)));
}

uint8_t ee_read_double(uint16_t adr, double *wert)    // acht byte lesen
{
  return(ee_read_n(adr, wert, sizeof(double)));
}

