#define G4_GPIO_TIMER

/*
2020-12-10
Michael Nowak
http://www.mino-elektronik.de
Alle Angaben ohne Gewaehr !
*/

#include "system_G431.h"

#define CALL_DELAY        70        // Verzgerung beim 1. Aufruf kompensieren

uint32_t  t15_cnt;                  // freier 1 ms Zhler
uint8_t   t15_tick;                 // flag fr 1 ms
uint16_t  led_cnt, gps_led_cnt;

volatile uint8_t akt_tasten;

// direktes Setzen oder Ruecksetzen eines Port-Pins
void set_pin(GPIO_TypeDef* GPIOx, uint8_t number, char set) {
  if(set) {
    GPIOx->BSRR = (SET_GPIO << number);       // IO-Bit setzen
  } else {
    GPIOx->BSRR = (RESET_GPIO << number);     // IO-Bit loeschen
  }
}

void delay_us(uint32_t us)
{
static uint32_t alter_takt;
  if(alter_takt != SystemCoreClock) {
    TIM4->CR1 = 0;                             // stoppen
    alter_takt = SystemCoreClock;
    RCC->APB1ENR1 |= RCC_APB1ENR1_TIM4EN;
    TIM4->ARR = (SystemCoreClock/MHZ_FAKTOR-1); // fr 1 s
    TIM4->CR1 |= TIM_CR1_CEN;                // T4 starten
  }  
  TIM4->CNT = CALL_DELAY;                    // Funktionsaufruf kompensieren
  TIM4->SR = 0;                              // berlauf flag lschen 
  while(us--) {
    while(!(TIM4->SR));                      // auf berlauf warten
    TIM4->SR = 0;                            // und Flags wieder lschen
  }  
}

void delay_ms(uint32_t ms)
{
  delay_us(ms*1000);
}  


// 1 kHz ISR mit T15 erzeugen
void init_t15(void)
{
static uint32_t alter_takt;
  if(alter_takt != SystemCoreClock) {
    TIM15->CR1 = 0;                             // stoppen
    alter_takt = SystemCoreClock;
    RCC->APB2ENR |= RCC_APB2ENR_TIM15EN;
    TIM15->ARR = 1000-1;                        // Teiler fr 1 kHz
    TIM15->PSC = SystemCoreClock/MHZ_FAKTOR-1;  // Vorteiler auf 1 MHz
    TIM15->DIER = TIM_DIER_UIE;                 // und Interrupt freigeben
    
    NVIC_EnableIRQ(TIM1_BRK_TIM15_IRQn);
    NVIC_SetPriority(TIM1_BRK_TIM15_IRQn,PRIO_TIM15);  // geringe Prioritaet
  }    
  TIM15->CR1 = TIM_CR1_CEN;                     // starten
} 

// Aufruf mit 1 kHz
void TIM1_BRK_TIM15_IRQHandler(void)              
{
static uint32_t decode_cnt;  
  TIM15->SR = 0;                              // flag loeschen
  t15_cnt++;                                  // freier ms-Zhler
  t15_tick = 1;

// fertig-LED und 1pps-LED blinken lassen
  if(led_cnt) {
    led_cnt--;
    set_pin(GPIOB, LED_FERTIG, 1);            // zur Kontrolle
  } else {
    set_pin(GPIOB, LED_FERTIG, 0);            // zur Kontrolle
  }
  if(gps_led_cnt) {
    gps_led_cnt--;
    set_pin(GPIOF, LED_1PPS, 1);              // zur Kontrolle
  } else {
    set_pin(GPIOF, LED_1PPS, 0);              // zur Kontrolle
  }
  
  if(decode_cnt++ >= 10) {
    if(!lcd_busy) {
      LCD_PORT->BSRR = LCD_D4 | LCD_D5 | LCD_D6; // auf 1 setzen
      
      LCD_PORT->MODER &= 
                  ~(CLR_MODER<<LCD_D4*2 | // LCD-Datenleitungen auf Eingang
                    CLR_MODER<<LCD_D5*2 | 
                    CLR_MODER<<LCD_D6*2 | 
                    CLR_MODER<<LCD_D7*2);
      decode_cnt = 0;
      delay_us(1);
      akt_tasten = (LCD_PORT->IDR >> LCD_DATA_OFFSET & 7) ^ 7;
      
      LCD_PORT->MODER |= 
                  (OUT<<LCD_D4*2 |        // LCD-Datenleitungen wieder auf Ausgang  
                   OUT<<LCD_D5*2 | 
                   OUT<<LCD_D6*2 | 
                   OUT<<LCD_D7*2);  
    }
  }  
  dekodiere_taste(akt_tasten);            // 1 ms Aufrufe werden auf 10 ms reduziert
  __ISB();                                // gelschte Flags abwarten
}    
